﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SpringPendulum
{
	public partial class MainPage : UserControl
	{
		// 画面の位置
		const int DISP_WIDTH = 400;
		const int DISP_HEIGHT = 400;

		int width, height;
		int cx, cy;

		// 各種パラメータ
		const int R_BALL = 12;   // ボールの直径
		double x1, y1; // ボールの座標
		const double dt = 0.001; //時間刻み

		const double M = 0.05;	// 質量
		const double G = 9.8; // 重力加速度
		const double K = 4.0;	// ばね定数

		double px;	// ばねの伸び
		double vx;	// ばねの伸び速度 dx/dt

		double th;	// ふれ角度
		double ah;	// ふれ角の速度 dθ/dt

		// 描画オブジェクト
		MyCircle ball;
		Line lines;

		public MainPage()
		{
			InitializeComponent();
			
			// 画面サイズを設定
			width = DISP_WIDTH;
			height = DISP_HEIGHT;
			cx = width / 2;
			cy = height / 2;

			initObjects();
			CompositionTarget.Rendering += new EventHandler(MainHandler);
		}

		// 変数の初期化
		private void initVars()
		{
			Random rnd = new System.Random();

			// 各種パラメータの初期化
			double th0 = rnd.Next(1, 180);	// ふれ角初期値
			th = th0 * Math.PI / 180.0;	// ふれ角度

			px = rnd.Next(60, 99) / 100;	// ばねの伸び
			vx = 0.0;	// ばねの伸び速度 dx/dt
		}

		// オブジェクトの初期化
		private void initObjects()
		{
			initVars();

			// ボールの位置を算出
			x1 = (1 + px) * Math.Sin(th);
			y1 = (1 + px) * Math.Cos(th);

			// 線を設置
			lines = new Line();
			lines.X1 = dispX(0.0);
			lines.Y1 = dispY(0.0);
			lines.X2 = dispX(x1);
			lines.Y2 = dispY(y1);
			lines.Stroke = new SolidColorBrush(Colors.Black);
			this.LayoutRoot.Children.Add(lines);
			
			// ボールを設置
			ball = new MyCircle(dispX(x1), dispY(y1), R_BALL);
			this.LayoutRoot.Children.Add(ball.obj);
		}

		// ボタンを押したらリスタート
		private void Do_btn1_Click(object sender, RoutedEventArgs e)
		{
			initVars();
		}

		// タイマーで呼び出されるメインハンドラー
		private void MainHandler(object o, EventArgs sender)
		{
			// ここでボールの移動を行う
			for (int i = 0; i < 10; i++) // 細かくステップを刻む
			{
				UpdateAction();
			}

			// ボールの位置を算出
			x1 = (1 + px) * Math.Sin(th);
			y1 = (1 + px) * Math.Cos(th);

			// ボールの描画
			ball.x = dispX(x1);
			ball.y = dispY(y1);
			ball.Action();

			// 線の描画
		//	lines.X1 = dispX(0.0);
		//	lines.Y1 = dispY(0.0);
			lines.X2 = dispX(x1);
			lines.Y2 = dispY(y1);
			
			// 数字の表示、動いていると何となくそれっぽい
			DebugText.Text
				= "(" + x1.ToString("F") + ", " + y1.ToString("F") + ")";
		}

		// ルンゲ・クッタ法で計算
		private void UpdateAction()
		{
			double p1 = vx;
			double q1 = ah;
			double r1 = diff1(px, th, vx, ah);
			double s1 = diff2(px, th, vx, ah);

			double p2 = vx + r1 / 2 * dt;
			double q2 = ah + s1 / 2 * dt;
			double r2 = diff1(px + p1 / 2 * dt, th + q1 / 2 * dt,
				vx + r1 / 2 * dt, ah + s1 / 2 * dt);
			double s2 = diff2(px + p1 / 2 * dt, th + q1 / 2 * dt,
				vx + r1 / 2 * dt, ah + s1 / 2 * dt);

			double p3 = vx + r2 / 2 * dt;
			double q3 = ah + s2 / 2 * dt;
			double r3 = diff1(px + p2 / 2 * dt, th + q2 / 2 * dt,
				vx + r2 / 2 * dt, ah + s2 / 2 * dt);
			double s3 = diff2(px + p2 / 2 * dt, th + q2 / 2 * dt,
				vx + r2 / 2 * dt, ah + s2 / 2 * dt);

			double p4 = vx + r3 * dt;
			double q4 = ah + s3 * dt;
			double r4 = diff1(px + p3 * dt, th + q3 * dt,
				vx + r3 * dt, ah + s3 * dt);
			double s4 = diff2(px + p3 * dt, th + q3 * dt,
				vx + r3 * dt, ah + s3 * dt);

			px = px + (p1 + 2 * p2 + 2 * p3 + p4) / 6 * dt;
			th = th + (q1 + 2 * q2 + 2 * q3 + q4) / 6 * dt;
			vx = vx + (r1 + 2 * r2 + 2 * r3 + r4) / 6 * dt;
			ah = ah + (s1 + 2 * s2 + 2 * s3 + s4) / 6 * dt;
		}

		private double diff1(double xx, double tt, double vv, double aa)
		{
			return (1 + xx) * aa * aa + G * Math.Cos(tt) - K / M * xx;
		}

		private double diff2(double xx, double tt, double vv, double aa)
		{
			return (-2 * vv * aa - G * Math.Sin(tt)) / (1 + xx);
		}

		// 画面座標への変換
		private double dispX(double x0)
		{
			return ((x0 + 2.0) * width / 4.0);
		}
		private double dispY(double y0)
		{
			return ((y0 + 2.0) * height / 4.0);
		}
	}

	/**
	 *  ボールオブジェクト
	 */
	class MyCircle
	{
		public Ellipse obj;
		public double r;
		public double x, y;

		public MyCircle(double x0, double y0, double r0)
		{
			this.obj = new Ellipse();
			this.r = r0;
			this.x = x0;
			this.y = y0;
			this.obj.Width = 2 * r0;
			this.obj.Height = 2 * r0;

			this.setColor(Color.FromArgb(0xFF, 0xFF, 0x99, 0x00)); //オレンジ色
		}

		public void setColor(Color c)
		{
			SolidColorBrush brush = new SolidColorBrush(c);
			this.obj.Fill = brush;
		}

		public void Action()
		{
			this.obj.SetValue(Canvas.LeftProperty, this.x - this.r);
			this.obj.SetValue(Canvas.TopProperty, this.y - this.r);
		}
	}
}
